package xylia.sax.contentmodelparser;

import org.xml.sax.ext.DeclHandler;
import org.xml.sax.SAXException ;
import java.io.* ;

/** A DeclHandler to handle content models.
 * <p>This is a filter in that events are passed to the next DeclHandler if any.</p>
 * <p>The elementDecl events cause the content model to be parsed and its parsed
 * representation is passed onto a single listener object of type ParsedContentModelHandler.
 * The representation of the content model is build by an object of type ContentModelFactory.</p>
 * <p>Typical usage of this class is as follows</p>
 * <pre>
 *     XMLReader xmlr ; 
 *     ...
 *     ContentModelMap hmap = new ContentModelMap() ;
 *     DeclHandler dh = new ContentModelParserDeclHandler( hmap ) ;
 *     xmlr.setProperty("http://xml.org/sax/properties/declaration-handler", dh);
 * </pre>
 * <p>Now, after parsing, hmap will be a Map from element names to DefaultContentModel.</p>
 * 
 * Copyright:    Copyright (c) 2001
 * Company:      MUN
 * @author   Theodore Norvell
 * @version 1.0
 */

public class ContentModelParserDeclHandler implements DeclHandler {

    private ContentModelFactory factory ;

    private DeclHandler nextHandler = null ;

    private ParsedContentModelHandler parsedContentModelHandler ;

    /** Construct with a DefaultContentModelFactory. 
     *  This means that the parsed content models will be of type DefaultContentModel
     *  @see DefaultContentModel
     *  @param parsedContentModelHandler the receiver of parsed content models.*/
    public ContentModelParserDeclHandler(ParsedContentModelHandler parsedContentModelHandler) {
        this(parsedContentModelHandler, new DefaultContentModelFactory() ) ; }

    /** Constuctor.
     *  @param parsedContentModelHandler The receiver of parsed content models
     *  @param factory An object that constructs the parsed content models.
     */
    public ContentModelParserDeclHandler( ParsedContentModelHandler parsedContentModelHandler, ContentModelFactory factory ) {
        this.parsedContentModelHandler = parsedContentModelHandler ;
        this.factory = factory ; }

    /** Set the next handler in the chain. */
    public void setNextHandler( DeclHandler nextHandler ) {
        this.nextHandler = nextHandler ; }
        
    /** Parses the content model and passes the parsed
     *  representation to the ParsedContentModelHandler.
     *  The event is then passed to the next DeclHandler.
     *  @param qname The qualified name of the object.
     *  @param contentModel The content model as a string. */
    public void elementDecl(String qname, String contentModel) throws SAXException {
        /*DBG System.out.println("qname is<"+qname+"> model is <"+contentModel+">");  /*DBG*/
        Reader reader = new StringReader( contentModel ) ;
        ContentModelParser parser = new ContentModelParser( reader ) ;
        parser.setFactory( factory ) ;
        try {
            Object parsedContentModel = parser.contentspec() ;
            parsedContentModelHandler.elementDecl( qname, parsedContentModel ) ; }
        catch( ParseException e ) {
            throw  new SAXException( "Error parsing content model for "+qname+"\n"+e.getMessage(), e ) ; }
        catch( TokenMgrError e ) {
            throw  new SAXException( "Error parsing content model for "+qname+"\n"+e.getMessage() ) ; }

        if( nextHandler != null ) {
            nextHandler.elementDecl( qname, contentModel ) ; }
    }

    /** The event is simply passed to the next DeclHandler. */
    public void attributeDecl(String parm1, String parm2, String parm3, String parm4, String parm5) throws SAXException {
        if( nextHandler != null ) {
            nextHandler.attributeDecl( parm1, parm2, parm3, parm4, parm5 ) ; }
    }

    /** The event is simply passed to the next DeclHandler. */
    public void internalEntityDecl(String parm1, String parm2) throws SAXException {
        if( nextHandler != null ) {
            nextHandler.internalEntityDecl( parm1, parm2 ) ; }
    }

    /** The event is simply passed to the next DeclHandler. */
    public void externalEntityDecl(String parm1, String parm2, String parm3) throws SAXException {
        if( nextHandler != null ) {
            nextHandler.externalEntityDecl( parm1, parm2, parm3 ) ; }
    }
}