package xylia.sax.contentmodelparser;

import xylia.utilities.Assert ;

/** Construct a DefaultContentModel.
 *  <p>This is a callback pbject for the parser that constructs
 *     DefaultContentModels.
 *  </p>>
 * Copyright:    Copyright (c) 2001
 * Company:      Memorial
 * @author Theodore Norvell, Dennis Peters and others.
 * @version 1.0
 */

public class DefaultContentModelFactory implements ContentModelFactory {

    /** Complete construction of a content model.
     *  @param kind, either ANY or EMPTY.
     *  @returns A DefaultContentModel.
     */
    public Object finishContentModel( int kind ) {
        return new DefaultContentModel( kind ) ; }

    /** Complete construction of a content model.
     *  @param kind Either MIXED or CHILDREN.
     *  @param regExp A regular expression representing the
     *  allowable sequence of children.
     *  @returns A DefaultContentModel.
     */
    public Object finishContentModel(  int kind, Object regExp ) {
        Assert.assert( regExp instanceof RegExp ) ;
        return new DefaultContentModel( kind, (RegExp) regExp ) ; }

    /** Create a regular expression object representing an occurence of #PCDATA in the content model.
     *  @returns An object representing the regular expression.
     */
    public Object mkPCDATA( ) {
        return new RegExp( RegExp.PCDATA ) ; }

    /** Create a regular expression object representing an occurence of an element name in the content model.
     *  @param name The element name.
     *  @returns An object representing the regular expression.
     */
    public Object mkName( String name ) {
        return new RegExp( RegExp.NAME, name ) ; }

    /** Create a regular expression object representing an occurence of an alternation in the content model.
     *  An alternation is a choice of two regular expressions: left | right
     *  @param left A regular expression object representing the left operand.
     *  @param right A regular expression object representing the right operand.
     *  @returns An object representing the regular expression.
     */
    public Object mkAlternation( Object right, Object left ) {
        Assert.assert( right instanceof RegExp ) ;
        Assert.assert( left instanceof RegExp ) ;
        return new RegExp( RegExp.ALT, (RegExp)right, (RegExp)left ) ; }

    /** Create a regular expression object representing an occurence of a sequence in the content model.
     *  A sequence is a succession of two regular expressions: left , right
     *  @param left A regular expression object representing the left operand.
     *  @param right A regular expression object representing the right operand.
     *  @returns An object representing the regular expression.
     */
    public Object mkSequence( Object right, Object left ) {
        Assert.assert( right instanceof RegExp ) ;
        Assert.assert( left instanceof RegExp ) ;
        return new RegExp( RegExp.SEQ, (RegExp)right, (RegExp)left ) ; }

    /** Create a regular expression object representing an occurence of optional regular expression in the content model.
     *  An optional regular expression represents something that may or may not
     *  be there: operand?
     *  @param operand A regular expression object representing the operand.
     *  @returns An object representing the regular expression.
     */
    public Object mkOptional( Object operand ) {
        Assert.assert( operand instanceof RegExp ) ;
        return new RegExp( RegExp.OPT, (RegExp) operand ) ; }

    /** Create a regular expression object representing possibly empty repetition of a regular expression in the content model.
     *  A possibly empty repetition of a regular expression represents something that may be repeated
     *  any number of times: operand*
     *  @param operand A regular expression object representing the operand.
     *  @returns An object representing the regular expression.
     */
    public Object mkKleeneStar( Object operand ) {
        Assert.assert( operand instanceof RegExp ) ;
        return new RegExp( RegExp.KSTAR, (RegExp) operand ) ; }

    /** Create a regular expression object representing nonempty repetition of a regular expression in the content model.
     *  A nonempty repetition of a regular expression represents something that may be repeated
     *  any number of times: operand+
     *  @param operand A regular expression object representing the operand.
     *  @returns An object representing the regular expression.
     */
    public Object mkKleenePlus( Object operand ) {
        Assert.assert( operand instanceof RegExp ) ;
        return new RegExp( RegExp.KPLUS, (RegExp) operand ) ; }
}